package net.cephas.blog;

import java.io.IOException;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;

import del.icio.us.Delicious;

/**
 * @author Aaron Johnson
 *
 */
public class DeliciousPoster {
	
	
	public static void main(String[] args) {
		
		// TODO: add ability to test configuration
		// TODO: add ability to specify a date
		// TODO: add ability to specify a timespan
		
		String properties;
		if (args.length < 1) {
			printUsage();
		} else {
			properties = args[0];
			DeliciousPosterConfig config = null; 
			try {
				config = DeliciousPosterConfig.parseProperties(properties);
			} catch (IOException e) {
				System.out.println("The properties file you specified could not be found.");
				System.exit(1);
			}
			
			int previousDays = -1;
			if (args.length > 1) {
				previousDays = Integer.parseInt(args[1]);
			}
			
			
			Calendar calendar = Calendar.getInstance();
			calendar.add(Calendar.DAY_OF_MONTH, previousDays);
			calendar.set(Calendar.HOUR_OF_DAY, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);

			Date yesterday = calendar.getTime();
			
			System.out.println(yesterday.toString());
			
			List entries = getEntries(config, yesterday);
			
			if (entries.size() > 0) {
				
				StringWriter sw = new StringWriter();
				try { 
					Velocity.init(); 
					VelocityContext context = new VelocityContext(); 
				
					context.put("posts", entries.toArray() );
					context.put("config", config);
				
					Template template = null; 
					template = Velocity.getTemplate(config.getVelocitytemplate());
					template.merge( context, sw );
					
				} catch( ResourceNotFoundException rnfe ) { 
					System.out.println("The Velocity template wasn't found."); 
				} catch( ParseErrorException pee ) { 
					System.out.println("The Velocity template couldn't be parsed: " + 
							pee.getMessage());
				} catch( MethodInvocationException mie ) { 
					System.out.println("The Velocity template execution failed."); 
				} catch( Exception e ) {
					System.out.println("The Velocity process failed.");
				}
				
				String title = "Links: " + (calendar.get(Calendar.MONTH) + 1) +
					"-" + calendar.get(Calendar.DATE) + 
					"-" + calendar.get(Calendar.YEAR);
				
				System.out.println(sw.toString());
				
				try {
					 postToMT(config, title, sw.toString());
				} catch (MalformedURLException e1) {
					System.out.println("The MT URL you used isn't valid.");
					e1.printStackTrace();
				} catch (IOException e1) {
					System.out.println("There was a problem contacting your MT server.");
				} catch (XmlRpcException e1) {
					System.out.println("There was a problem contacting your XML-RPC service.");
				}
				
			}
		}
	}
	
	public static List getEntries(DeliciousPosterConfig config, Date date) {
		Delicious yummy = new Delicious(config.getDelicioususername(), config.getDeliciouspassword());
		return yummy.getPostsForDate(null, date);
	}
	
	
	public static void postToMT(DeliciousPosterConfig config, String title, String body)
		throws MalformedURLException, IOException, XmlRpcException {
		
		XmlRpcClient xmlrpc = new XmlRpcClient(config.getBlogurl());
		Hashtable structContents=new Hashtable();
		structContents.put("title",title);
		structContents.put("description",body);
		Boolean publish = new Boolean(true);
	
		Vector params = new Vector();
		params.add(config.getBlogid());
		params.add(config.getBlogusername());
		params.add(config.getBlogpassword());
		params.add(structContents);
		params.add(publish); 
		Object message = xmlrpc.execute("metaWeblog.newPost",params);
		System.out.println(message.toString());
		
	}
	
	public static void printUsage() {
		String linesep = "-------------------------------------";
		System.out.println(
				"\n\n" +
				"Configuring DeliciousPoster:\n" + linesep + "\n" +
				"Edit the included config.properties file so that it includes " + 
				"your Movable Type username / password / URL and Blog ID and " + 
				"your del.icio.us username and password. Optionally edit the " +
				"included Velocity template or create your own and update the" +
				"path to it in config.properties.\n\n" +
				"Using DeliciousPoster:\n" + linesep + "\n" +
				"Windows:\n" + 
				"java -cp lib\\commons-codec-1.3.jar;lib\\commons-httpclient-3.0-rc2.jar;" +
				"lib\\commons-logging-1.0.4.jar;lib\\delicious-1.6.jar;" +
				"lib\\xmlrpc-1.2-b1.jar;lib\\commons-collections-3.1.jar;" +
				"lib\\log4j-1.2.11.jar;. net.cephas.blog.DeliciousPoster $x $y" +
				"\n\n" + 
				"Linux:\n" + 
				"java -cp lib/commons-codec-1.3.jar:lib/commons-httpclient-3.0-rc2.jar:" +
				"lib/commons-logging-1.0.4.jar:lib/delicious-1.6.jar:" +
				"lib/xmlrpc-1.2-b1.jar:lib/commons-collections-3.1.jar:" +
				"lib/log4j-1.2.11.jar:. net.cephas.blog.DeliciousPoster $x $y" +
				"\n\n" +
				"where $x is the name of your configuration document (the sample " +
				"provided is called config.properties) and $y is a number less than " +
				"or equal to zero that indicates how many days previous you'd like to " +
				"go to retrieve your del.icio.us feed (ie: -1 is yesterday, -7 is last " +
				"week, etc)."
				);
		
	}

	
	

	
}
